# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt

"""
.. module:: json_backend
   :synopsis: JSON file-based storage backend.
"""

import shutil
import os
from logging import getLogger
try:
    import json
except ImportError:  # pragma: no cover
    import simplejson as json

from base_backend import BackendIOException

log = getLogger(__name__)

class JsonBackend(object):
    """JSON file-based storage backend."""

    def __init__(self, directory, users_fname='users',
            roles_fname='roles', pending_reg_fname='register', initialize=False):
        """Data storage class. Handles JSON files

        :param users_fname: users file name (without .json)
        :type users_fname: str.
        :param roles_fname: roles file name (without .json)
        :type roles_fname: str.
        :param pending_reg_fname: pending registrations file name (without .json)
        :type pending_reg_fname: str.
        :param initialize: create empty JSON files (defaults to False)
        :type initialize: bool.
        """
        assert directory, "Directory name must be valid"
        self._directory = directory
        self.users = {}
        self._users_fname = users_fname
        self.roles = {}
        self._roles_fname = roles_fname
        self._mtimes = {}
        self._pending_reg_fname = pending_reg_fname
        self.pending_registrations = {}
        if initialize:
            self._initialize_storage()
        self._refresh()  # load users and roles

    def _initialize_storage(self):
        """Create empty JSON files"""
        self._savejson(self._users_fname, {})
        self._savejson(self._roles_fname, {})
        self._savejson(self._pending_reg_fname, {})

    def _refresh(self):
        """Load users and roles from JSON files, if needed"""
        self._loadjson(self._users_fname, self.users)
        self._loadjson(self._roles_fname, self.roles)
        self._loadjson(self._pending_reg_fname, self.pending_registrations)

    def _loadjson(self, fname, dest):
        """Load JSON file located under self._directory, if needed

        :param fname: short file name (without path and .json)
        :type fname: str.
        :param dest: destination
        :type dest: dict
        """
        try:
            fname = "%s/%s.json" % (self._directory, fname)
            mtime = os.stat(fname).st_mtime

            if self._mtimes.get(fname, 0) == mtime:
                # no need to reload the file: the mtime has not been changed
                return

            with open(fname) as f:
                json_data = f.read()
        except Exception as e:
            raise BackendIOException("Unable to read json file %s: %s" % (fname, e))

        try:
            json_obj = json.loads(json_data)
            dest.clear()
            dest.update(json_obj)
            self._mtimes[fname] = os.stat(fname).st_mtime
        except Exception as e:
            raise BackendIOException("Unable to parse JSON data from %s: %s" \
                % (fname, e))

    def _savejson(self, fname, obj):
        """Save obj in JSON format in a file in self._directory"""
        fname = "%s/%s.json" % (self._directory, fname)
        try:
            s = json.dumps(obj)
            with open("%s.tmp" % fname, 'wb') as f:
                f.write(s)
                f.flush()
            shutil.move("%s.tmp" % fname, fname)
        except Exception as e:
            raise BackendIOException("Unable to save JSON file %s: %s" \
                % (fname, e))

    def save_users(self):
        """Save users in a JSON file"""
        self._savejson(self._users_fname, self.users)

    def save_roles(self):
        """Save roles in a JSON file"""
        self._savejson(self._roles_fname, self.roles)

    def save_pending_registrations(self):
        """Save pending registrations in a JSON file"""
        self._savejson(self._pending_reg_fname, self.pending_registrations)



# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt

"""
.. module:: sqlite_backend
   :synopsis: SQLite storage backend.
"""

import base_backend
from logging import getLogger
log = getLogger(__name__)


class SqlRowProxy(dict):
    def __init__(self, table, key, row):
        li = ((k, v) for (k, ktype), v in zip(table._columns[1:], row[1:]))
        dict.__init__(self, li)
        self._table = table
        self._key = key

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, value)
        self._table[self._key] = self


class Table(base_backend.Table):
    """Provides dictionary-like access to an SQL table."""

    def __init__(self, backend, table_name):
        self._backend = backend
        self._engine = backend.connection
        self._table_name = table_name
        self._column_names = zip(*self._columns)[0]
        self._key_col_num = 0
        self._key_col_name = self._column_names[self._key_col_num]
        self._key_col = self._column_names[self._key_col_num]

    def _row_to_value(self, key, row):
        assert isinstance(row, tuple)
        row_key = row[self._key_col_num]
        row_value = SqlRowProxy(self, key, row)
        return row_key, row_value

    def __len__(self):
        query = "SELECT count() FROM %s" % self._table_name
        ret = self._backend.run_query(query)
        return ret.fetchone()[0]

    def __contains__(self, key):
        #FIXME: count()
        query = "SELECT * FROM %s WHERE %s='%s'" % \
            (self._table_name, self._key_col, key)
        row = self._backend.fetch_one(query)
        return row is not None

    def __setitem__(self, key, value):
        """Create or update a row"""
        assert isinstance(value, dict)
        v, cn = set(value), set(self._column_names[1:])
        assert not v - cn, repr(v - cn)
        assert not cn - v, repr(cn - v)

        assert set(value) == set(self._column_names[1:]), "%s %s" % \
            (repr(set(value)), repr(set(self._column_names[1:])))

        col_values = [key] + [value[k] for k in self._column_names[1:]]

        col_names = ', '.join(self._column_names)
        question_marks = ', '.join('?' for x in col_values)
        query = "INSERT OR REPLACE INTO %s (%s) VALUES (%s)" % \
            (self._table_name, col_names, question_marks)

        ret = self._backend.run_query_using_conversion(query, col_values)


    def __getitem__(self, key):
        query = "SELECT * FROM %s WHERE %s='%s'" % \
            (self._table_name, self._key_col, key)
        row = self._backend.fetch_one(query)
        if row is None:
            raise KeyError(key)

        return self._row_to_value(key, row)[1]
        #return dict(zip(self._column_names, row))

    def __iter__(self):
        """Iterate over table index key values"""
        query = "SELECT %s FROM %s" % (self._key_col, self._table_name)
        result = self._backend.run_query(query)
        for row in result:
            yield row[0]

    def iteritems(self):
        """Iterate over table rows"""
        query = "SELECT * FROM %s" % self._table_name
        result = self._backend.run_query(query)
        for row in result:
            d = dict(zip(self._column_names, row))
            d.pop(self._key_col)

            yield (self._key_col, d)

    def pop(self, key):
        d = self.__getitem__(key)
        query = "DELETE FROM %s WHERE %s='%s'" % \
            (self._table_name, self._key_col, key)
        self._backend.fetch_one(query)
        #FIXME: check deletion
        return d

    def insert(self, d):
        raise NotImplementedError

    def empty_table(self):
        raise NotImplementedError

    def create_table(self):
        """Issue table creation"""
        cc = []
        for col_name, col_type in self._columns:
            if col_type == int:
                col_type = 'INTEGER'
            elif col_type == str:
                col_type = 'TEXT'

            if col_name == self._key_col:
                extras = 'PRIMARY KEY ASC'
            else:
                extras = ''

            cc.append("%s %s %s" % (col_name, col_type, extras))

        cc = ','.join(cc)
        query = "CREATE TABLE %s (%s)" % (self._table_name, cc)
        self._backend.run_query(query)


class SingleValueTable(Table):
    def __init__(self, *args):
        super(SingleValueTable, self).__init__(*args)
        self._value_col = self._column_names[1]

    def __setitem__(self, key, value):
        """Create or update a row"""
        assert not isinstance(value, dict)
        query = "INSERT OR REPLACE INTO %s (%s, %s) VALUES (?, ?)" % \
            (self._table_name, self._key_col, self._value_col)

        col_values = (key, value)
        ret = self._backend.run_query_using_conversion(query, col_values)

    def __getitem__(self, key):
        query = "SELECT %s FROM %s WHERE %s='%s'" % \
            (self._value_col, self._table_name, self._key_col, key)
        row = self._backend.fetch_one(query)
        if row is None:
            raise KeyError(key)

        return row[0]

class UsersTable(Table):
    def __init__(self, *args, **kwargs):
        self._columns = (
            ('username', str),
            ('role', str),
            ('hash', str),
            ('email_addr', str),
            ('desc', str),
            ('creation_date', str),
            ('last_login', str)
        )
        super(UsersTable, self).__init__(*args, **kwargs)

class RolesTable(SingleValueTable):
    def __init__(self, *args, **kwargs):
        self._columns = (
            ('role', str),
            ('level', int)
        )
        super(RolesTable, self).__init__(*args, **kwargs)

class PendingRegistrationsTable(Table):
    def __init__(self, *args, **kwargs):
        self._columns = (
            ('code', str),
            ('username', str),
            ('role', str),
            ('hash', str),
            ('email_addr', str),
            ('desc', str),
            ('creation_date', str)
        )
        super(PendingRegistrationsTable, self).__init__(*args, **kwargs)




class SQLiteBackend(base_backend.Backend):

    def __init__(self, filename, users_tname='users', roles_tname='roles',
            pending_reg_tname='register', initialize=False):

        self._filename = filename

        self.users = UsersTable(self, users_tname)
        self.roles = RolesTable(self, roles_tname)
        self.pending_registrations = PendingRegistrationsTable(self, pending_reg_tname)

        if initialize:
            self.users.create_table()
            self.roles.create_table()
            self.pending_registrations.create_table()
            log.debug("Tables created")

    @property
    def connection(self):
        try:
            return self._connection
        except AttributeError:
            import sqlite3
            self._connection = sqlite3.connect(self._filename)
            return self._connection

    def run_query(self, query):
        return self._connection.execute(query)

    def run_query_using_conversion(self, query, args):
        return self._connection.execute(query, args)

    def fetch_one(self, query):
        return self._connection.execute(query).fetchone()

    def _initialize_storage(self, db_name):
        raise NotImplementedError

    def _drop_all_tables(self):
        raise NotImplementedError

    def save_users(self): pass
    def save_roles(self): pass
    def save_pending_registrations(self): pass



# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Unit testing - test the Cork module against a real MongoDB instance
# running on localhost.

from base64 import b64encode, b64decode
from nose import SkipTest
from nose.tools import assert_raises, raises, with_setup
from time import time
import bottle
import mock
import os
import shutil

from cork import Cork, AAAException, AuthException
from cork.backends import MongoDBBackend
import testutils

testdir = None  # Test directory

class Conf(object):

    def setup_test_db(self):
        mb = MongoDBBackend(db_name='cork-functional-test', initialize=True)

        # Purge DB
        mb.users._coll.drop()
        mb.roles._coll.drop()
        mb.pending_registrations._coll.drop()

        # Create admin
        mb.users._coll.insert({
            "login": "admin",
            "email_addr": "admin@localhost.local",
            "desc": "admin test user",
            "role": "admin",
            "hash": "cLzRnzbEwehP6ZzTREh3A4MXJyNo+TV8Hs4//EEbPbiDoo+dmNg22f2RJC282aSwgyWv/O6s3h42qrA6iHx8yfw=",
            "creation_date": "2012-10-28 20:50:26.286723"
        })

        # Create users
        mb.roles._coll.insert({'role': 'special', 'val': 200})
        mb.roles._coll.insert({'role': 'admin', 'val': 100})
        mb.roles._coll.insert({'role': 'editor', 'val': 60})
        mb.roles._coll.insert({'role': 'user', 'val': 50})

        return mb

    def purge_test_db(self):
        # Purge DB
        mb = MongoDBBackend(db_name='cork-functional-test', initialize=True)
        mb.users._coll.drop()
        mb.roles._coll.drop()


    def test_iteritems_on_users(self):
        for k, v in self.aaa._store.users.iteritems():
            #assert isinstance(k, str)
            #assert isinstance(v, dict)
            expected_dkeys = set(('hash', 'email_addr', 'role', 'creation_date',
                'desc'))
            dkeys = set(v.keys())

            extra = dkeys - expected_dkeys
            assert not extra, "Unexpected extra keys: %s" % repr(extra)

            missing = expected_dkeys - dkeys
            assert not missing, "Missing keys: %s" % repr(missing)


class TestMongoDBUnauth(Conf, testutils.DatabaseInteractionAsUnauthenticated):
    pass

class TestMongoDBAdmin(Conf, testutils.DatabaseInteractionAsAdmin):
    pass



# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Unit testing - test the Cork module against a Json-based db using the
# JsonBackend backend module

from nose import SkipTest
from nose.tools import assert_raises, raises, with_setup
from time import time
import os
import shutil

from cork import Cork, AAAException, AuthException
from cork.backends import JsonBackend
import testutils

class Conf(object):

    def setup_test_db(self):
        """Setup test directory with valid JSON files"""
        testdir = testutils.pick_temp_directory()
        os.mkdir(testdir + '/views')
        with open("%s/users.json" % testdir, 'w') as f:
            f.write("""{"admin": {"email_addr": "admin@localhost.local", "desc": null, "role": "admin", "hash": "69f75f38ac3bfd6ac813794f3d8c47acc867adb10b806e8979316ddbf6113999b6052efe4ba95c0fa9f6a568bddf60e8e5572d9254dbf3d533085e9153265623", "creation_date": "2012-04-09 14:22:27.075596"}}""")
        with open("%s/roles.json" % testdir, 'w') as f:
            f.write("""{"special": 200, "admin": 100, "user": 50, "editor": 60}""")
        with open("%s/register.json" % testdir, 'w') as f:
            f.write("""{}""")
        with open("%s/views/registration_email.tpl" % testdir, 'w') as f:
            f.write("""Username:{{username}} Email:{{email_addr}} Code:{{registration_code}}""")
        with open("%s/views/password_reset_email.tpl" % testdir, 'w') as f:
            f.write("""Username:{{username}} Email:{{email_addr}} Code:{{reset_code}}""")
        print("setup done in %s" % testdir)

        #jb = JsonBackend(testdir, initialize=True)
        jb = JsonBackend(testdir)

        self.__testdir = testdir
        return jb

    def purge_test_db(self):
        global cookie_name
        cookie_name = None
        if self.__testdir:
            shutil.rmtree(self.__testdir)
            self.__testdir = None

    def test_iteritems_on_users(self):
        for k, v in self.aaa._store.users.iteritems():
            expected_dkeys = set(('hash', 'email_addr', 'role', 'creation_date',
                'desc'))
            dkeys = set(v.keys())

            extra = dkeys - expected_dkeys
            assert not extra, "Unexpected extra keys: %s" % repr(extra)

            missing = expected_dkeys - dkeys
            assert not missing, "Missing keys: %s" % repr(missing)

class TestUnauth(Conf, testutils.DatabaseInteractionAsUnauthenticated):
    pass

class TestAdmin(Conf, testutils.DatabaseInteractionAsAdmin):
    pass



# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt

"""
.. module:: backends
   :synopsis: Backends API - used to make backends available for importing
"""

from json_backend import JsonBackend
from mongodb_backend import MongoDBBackend
from sqlalchemy_backend import SqlAlchemyBackend
from sqlite_backend import SQLiteBackend






# content of conftest.py
import pytest

class A(object):
    pass

@pytest.fixture
def base():
    print 'INST A'
    return A()

@pytest.fixture(params=["merlinux.eu", "mail.python.org"])
def smtp(request, base):
    print request.param
    return base

@pytest.fixture(params=['user', 'admin', None])
def auth_status(request, base):
    return base


def test_nyan(smtp, auth_status):
    assert smtp == auth_status



# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Unit testing - test the Cork module against an in-memory SQLite DB using
# the SQLiteBackend backend module.

from nose import SkipTest
from nose.tools import assert_raises, raises, with_setup

from cork import Cork, AAAException, AuthException
from cork.backends import SQLiteBackend
import testutils

class Conf(object):

    def setup_test_db(self):
        b = SQLiteBackend(':memory:', initialize=True)
        b.connection.executescript("""
            INSERT INTO users (username, email_addr, desc, role, hash, creation_date) VALUES
            (
                'admin',
                'admin@localhost.local',
                'admin test user',
                'admin',
                'cLzRnzbEwehP6ZzTREh3A4MXJyNo+TV8Hs4//EEbPbiDoo+dmNg22f2RJC282aSwgyWv/O6s3h42qrA6iHx8yfw=',
                '2012-10-28 20:50:26.286723'
            );
            INSERT INTO roles (role, level) VALUES ('special', 200);
            INSERT INTO roles (role, level) VALUES ('admin', 100);
            INSERT INTO roles (role, level) VALUES ('editor', 60);
            INSERT INTO roles (role, level) VALUES ('user', 50);
        """)
        return b

class TestSQLiteUnauth(Conf, testutils.DatabaseInteractionAsUnauthenticated):
    pass

class TestSQLiteAdmin(Conf, testutils.DatabaseInteractionAsAdmin):
    pass








# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Unit testing - utility functions.
#
import bottle
import os
import shutil
import sys
import json
import mock
import tempfile
from base64 import b64encode, b64decode
from nose.tools import assert_raises, raises, with_setup
from nose import SkipTest

from datetime import datetime, timedelta
from cork import Cork, AAAException, AuthException

cookie_name = None

def pick_temp_directory():
    """Select a temporary directory for the test files.
    Set the tmproot global variable.
    """
    if os.environ.get('TRAVIS', False):
        return tempfile.mkdtemp()

    if sys.platform == 'linux2':
        # In-memory filesystem allows faster testing.
        return tempfile.mkdtemp(dir='/dev/shm')

    return tempfile.mkdtemp()


def purge_temp_directory(test_dir):
    """Remove the test directory"""
    assert test_dir
    shutil.rmtree(test_dir)

def assert_is_redirect(e, path):
    """Check if an HTTPResponse is a redirect.

    :param path: relative path without leading slash.
    :type path: str
    """
    assert isinstance(e, bottle.HTTPResponse), "Incorrect exception type passed to assert_is_redirect"
    assert e.status_code == 302, "HTTPResponse status should be 302 but is '%s'" % e.status
    redir_location = e.headers['Location'].rsplit('/', 1)[1]
    assert redir_location == path, "Redirected to %s instead of %s" % (redir_location, path)







class RoAttrDict(dict):
    """Read-only attribute-accessed dictionary.
    Used to mock beaker's session objects
    """
    def __getattr__(self, name):
        return self[name]

    def delete(self):
        """Used during logout to delete the current session"""
        global cookie_name
        cookie_name = None



class MockedAdminCork(Cork):
    """Mocked module where the current user is always 'admin'"""
    @property
    def _beaker_session(self):
        return RoAttrDict(username='admin')

    def _setup_cookie(self, username):
        global cookie_name
        cookie_name = username


class MockedUnauthenticatedCork(Cork):
    """Mocked module where the current user not set"""
    @property
    def _beaker_session(self):
        return RoAttrDict()

    def _setup_cookie(self, username):
        cookie_name = username



class DatabaseInteractionAsUnauthenticated(object):

    def setUp(self):
        """Setup test directory and a MockedAdminCork instance"""
        global cookie_name
        print 'Performing MockedUnauthenticatedCork setup'
        mb = self.setup_test_db()
        self.aaa = MockedUnauthenticatedCork(backend=mb,
            smtp_server='localhost', email_sender='test@localhost')
        cookie_name = None

    def tearDown(self):
        if hasattr(self, 'purge_test_db'):
            self.purge_test_db()

        del(self.aaa)
        cookie_name = None

    @raises(AAAException)
    def test_get_current_user_unauth(self):
        print self.aaa.current_user
        self.aaa.current_user['username']

    def test_unauth_is_anonymous(self):
        assert self.aaa.user_is_anonymous




class DatabaseInteractionAsAdmin(object):

    def setUp(self):
        """Setup test database and a MockedAdminCork instance"""
        global cookie_name
        mb = self.setup_test_db()
        print 'Performing MockedAdminCork setup'
        self.aaa = MockedAdminCork(backend=mb, smtp_server='localhost',
            email_sender='test@localhost')
        cookie_name = None


    def tearDown(self):
        if hasattr(self, 'purge_test_db'):
            self.purge_test_db()

        del(self.aaa)
        cookie_name = None


    def test_mockedadmin(self):
        assert len(self.aaa._store.users) == 1,  len(self.aaa._store.users)
        assert 'admin' in self.aaa._store.users, repr(self.aaa._store.users)


    def test_password_hashing(self):
        shash = self.aaa._hash('user_foo', 'bogus_pwd')
        assert len(shash) == 88, "hash length should be 88 and is %d" % len(shash)
        assert shash.endswith('='), "hash should end with '='"
        assert self.aaa._verify_password('user_foo', 'bogus_pwd', shash) == True, \
            "Hashing verification should succeed"


    def test_incorrect_password_hashing(self):
        shash = self.aaa._hash('user_foo', 'bogus_pwd')
        assert len(shash) == 88, "hash length should be 88 and is %d" % len(shash)
        assert shash.endswith('='), "hash should end with '='"
        assert self.aaa._verify_password('user_foo', '####', shash) == False, \
            "Hashing verification should fail"
        assert self.aaa._verify_password('###', 'bogus_pwd', shash) == False, \
            "Hashing verification should fail"


    def test_password_hashing_collision(self):
        salt = 'S' * 32
        hash1 = self.aaa._hash('user_foo', 'bogus_pwd', salt=salt)
        hash2 = self.aaa._hash('user_foobogus', '_pwd', salt=salt)
        assert hash1 != hash2, "Hash collision"


    def test_unauth_create_role(self):
        assert len(self.aaa._store.users) == 1, "Only the admin user should be present"
        self.aaa._store.roles['admin'] = 10  # lower admin level
        assert self.aaa._store.roles['admin'] == 10, self.aaa._store.roles['admin']
        assert len(self.aaa._store.users) == 1, "Only the admin user should be present"
        assert_raises(AuthException, self.aaa.create_role, 'user', 33)


    def test_create_existing_role(self):
        assert_raises(AAAException, self.aaa.create_role, 'user', 33)

    @raises(KeyError)
    def test_access_nonexisting_role(self):
        self.aaa._store.roles['NotThere']

    @raises(AAAException)
    def test_create_role_with_incorrect_level(self):
        self.aaa.create_role('new_user', 'not_a_number')


    def test_create_role(self):
        assert len(self.aaa._store.roles) == 4, repr(self.aaa._store.roles)
        self.aaa.create_role('user33', 33)
        assert len(self.aaa._store.roles) == 5, repr(self.aaa._store.roles)


    def test_unauth_delete_role(self):
        self.aaa._store.roles['admin'] = 10  # lower admin level
        assert_raises(AuthException, self.aaa.delete_role, 'user')


    def test_delete_nonexistent_role(self):
        assert_raises(AAAException, self.aaa.delete_role, 'user123')


    def test_create_delete_role(self):
        assert len(self.aaa._store.roles) == 4, repr(self.aaa._store.roles)
        self.aaa.create_role('user33', 33)
        assert len(self.aaa._store.roles) == 5, repr(self.aaa._store.roles)

        assert self.aaa._store.roles['user33'] == 33
        self.aaa.delete_role('user33')
        assert len(self.aaa._store.roles) == 4, repr(self.aaa._store.roles)


    def test_list_roles(self):
        roles = list(self.aaa.list_roles())
        assert len(roles) == 4, "Incorrect. Users are: %s" % repr(self.aaa._store.roles)


    def test_unauth_create_user(self):
        self.aaa._store.roles['admin'] = 10  # lower admin level
        assert_raises(AuthException, self.aaa.create_user, 'phil', 'user', 'hunter123')


    def test_create_existing_user(self):
        assert_raises(AAAException, self.aaa.create_user, 'admin', 'admin', 'bogus')


    @raises(AAAException)
    def test_create_user_with_wrong_role(self):
        self.aaa.create_user('admin2', 'nonexistent_role', 'bogus')


    def test_create_user(self):
        assert len(self.aaa._store.users) == 1, repr(self.aaa._store.users)
        self.aaa.create_user('phil', 'user', 'user')
        assert len(self.aaa._store.users) == 2, repr(self.aaa._store.users)
        assert 'phil' in self.aaa._store.users


    def test_unauth_delete_user(self):
        self.aaa._store.roles['admin'] = 10  # lower admin level
        assert_raises(AuthException, self.aaa.delete_user, 'phil')


    def test_delete_nonexistent_user(self):
        assert_raises(AAAException, self.aaa.delete_user, 'not_an_user')


    def test_delete_user(self):
        assert len(self.aaa._store.users) == 1, repr(self.aaa._store.users)
        self.aaa.delete_user('admin')
        assert len(self.aaa._store.users) == 0, repr(self.aaa._store.users)
        assert 'admin' not in self.aaa._store.users



    def test_list_users(self):
        users = list(self.aaa.list_users())
        assert len(users) == 1, "Incorrect. Users are: %s" % repr(self.aaa._store.users)

    def test_iteritems_on_users(self):
        for k, v in self.aaa._store.users.iteritems():
            #assert isinstance(k, str)
            #assert isinstance(v, dict)
            expected_dkeys = set(('hash', 'email_addr', 'role', 'creation_date',
                'desc', 'last_login'))
            dkeys = set(v.keys())

            extra = dkeys - expected_dkeys
            assert not extra, "Unexpected extra keys: %s" % repr(extra)

            missing = expected_dkeys - dkeys
            assert not missing, "Missing keys: %s" % repr(missing)


    def test_failing_login(self):
        login = self.aaa.login('phil', 'hunter123')
        assert login == False, "Login must fail"
        assert cookie_name == None


    def test_login_nonexistent_user_empty_password(self):
        login = self.aaa.login('IAmNotHome', '')
        assert login == False, "Login must fail"
        assert cookie_name == None


    def test_login_existing_user_empty_password(self):
        self.aaa.create_user('phil', 'user', 'hunter123')
        assert 'phil' in self.aaa._store.users
        assert self.aaa._store.users['phil']['role'] == 'user'
        login = self.aaa.login('phil', '')
        assert login == False, "Login must fail"
        assert cookie_name == None


    def test_create_and_validate_user(self):
        assert len(self.aaa._store.users) == 1, "Only the admin user should be present"
        self.aaa.create_user('phil', 'user', 'hunter123')
        assert len(self.aaa._store.users) == 2, "Two users should be present"
        assert 'phil' in self.aaa._store.users
        assert self.aaa._store.users['phil']['role'] == 'user'
        login = self.aaa.login('phil', 'hunter123')
        assert login == True, "Login must succeed"
        assert cookie_name == 'phil'

    def test_create_user_login_logout(self):
        assert 'phil' not in self.aaa._store.users
        self.aaa.create_user('phil', 'user', 'hunter123')
        assert 'phil' in self.aaa._store.users
        login = self.aaa.login('phil', 'hunter123')
        assert login == True, "Login must succeed"
        assert cookie_name == 'phil'
        try:
            self.aaa.logout(fail_redirect='/failed_logout')
        except bottle.HTTPResponse, e:
            assert_is_redirect(e, 'login')

        assert cookie_name == None

    def test_modify_user_using_overwrite(self):
        self.aaa.create_user('phil', 'user', 'hunter123')
        assert 'phil' in self.aaa._store.users
        u = self.aaa._store.users['phil']
        u.update(role='editor')
        self.aaa._store.users['phil'] = u
        assert self.aaa._store.users['phil']['role'] == 'editor'

    def test_modify_user(self):
        self.aaa.create_user('phil', 'user', 'hunter123')
        self.aaa._store.users['phil']['role'] = 'editor'
        assert self.aaa._store.users['phil']['role'] == 'editor', self.aaa._store.users['phil']


    def test_modify_user_using_local_change(self):
        self.aaa.create_user('phil', 'user', 'hunter123')
        u = self.aaa._store.users['phil']
        u['role'] = 'editor'
        assert u['role'] == 'editor', repr(u)
        assert self.aaa._store.users['phil']['role'] == 'editor'


    def test_require_failing_username(self):
        # The user exists, but I'm 'admin'
        self.aaa.create_user('phil', 'user', 'hunter123')
        assert_raises(AuthException, self.aaa.require, username='phil')


    def test_require_nonexistent_username(self):
        assert_raises(AAAException, self.aaa.require, username='no_such_user')


    def test_require_failing_role_fixed(self):
        assert_raises(AuthException, self.aaa.require, role='user', fixed_role=True)


    @raises(AAAException)
    def test_require_missing_parameter(self):
        self.aaa.require(fixed_role=True)


    def test_require_nonexistent_role(self):
        assert_raises(AAAException, self.aaa.require, role='clown')

    def test_require_failing_role(self):
        # Requesting level >= 100
        assert_raises(AuthException, self.aaa.require, role='special')


    def test_successful_require_role(self):
        self.aaa.require(username='admin')
        self.aaa.require(username='admin', role='admin')
        self.aaa.require(username='admin', role='admin', fixed_role=True)
        self.aaa.require(username='admin', role='user')


    def test_authenticated_is_not_anonymous(self):
        assert not self.aaa.user_is_anonymous


    def test_update_nonexistent_role(self):
        assert_raises(AAAException, self.aaa.current_user.update, role='clown')


    @raises(AAAException)
    def test_update_nonexistent_user(self):
        self.aaa._store.users.pop('admin')
        self.aaa.current_user.update(role='user')


    def test_update_role(self):
        self.aaa.current_user.update(role='user')
        assert self.aaa._store.users['admin']['role'] == 'user'


    def test_update_pwd(self):
        self.aaa.current_user.update(pwd='meow')


    def test_update_email(self):
        print self.aaa._store.users['admin']
        self.aaa.current_user.update(email_addr='foo')
        assert self.aaa._store.users['admin']['email_addr'] == 'foo', self.aaa._store.users['admin']


    @raises(AuthException)
    def test_get_current_user_nonexistent(self):
        # The current user 'admin' is not in the user table
        self.aaa._store.users.pop('admin')
        self.aaa.current_user


    def test_get_nonexistent_user(self):
        assert self.aaa.user('nonexistent_user') is None


    def test_get_user_description_field(self):
        admin = self.aaa.user('admin')
        for field in ['description', 'email_addr']:
            assert field in admin.__dict__


    def test_register_no_user(self):
        assert_raises(AssertionError, self.aaa.register, None, 'pwd', 'a@a.a')


    def test_register_no_pwd(self):
        assert_raises(AssertionError, self.aaa.register, 'foo', None, 'a@a.a')


    def test_register_no_email(self):
        assert_raises(AssertionError, self.aaa.register, 'foo', 'pwd', None)


    def test_register_already_existing(self):
        assert_raises(AAAException, self.aaa.register, 'admin', 'pwd', 'a@a.a')


    def test_register_no_role(self):
        assert_raises(AAAException, self.aaa.register, 'foo', 'pwd', 'a@a.a', role='clown')


    def test_register_role_too_high(self):
        assert_raises(AAAException, self.aaa.register, 'foo', 'pwd', 'a@a.a', role='admin')


    def test_register_valid(self):
        self.aaa.mailer.send_email = mock.Mock()
        self.aaa.register('foo', 'pwd', 'email@email.org', role='user',
            email_template='examples/views/registration_email.tpl'
        )
        assert self.aaa.mailer.send_email.called
        r = self.aaa._store.pending_registrations
        assert len(r) == 1
        reg_code = list(r)[0]
        assert r[reg_code]['username'] == 'foo'
        assert r[reg_code]['email_addr'] == 'email@email.org'
        assert r[reg_code]['role'] == 'user'


    def test_validate_registration_no_code(self):
        assert_raises(AAAException, self.aaa.validate_registration, 'not_a_valid_code')

    def test_validate_registration(self):
        self.aaa.mailer.send_email = mock.Mock()
        self.aaa.register('foo', 'pwd', 'email@email.org', role='user',
            email_template='examples/views/registration_email.tpl'
        )
        r = self.aaa._store.pending_registrations
        reg_code = list(r)[0]

        assert len(self.aaa._store.users) == 1, "Only the admin user should be present"
        self.aaa.validate_registration(reg_code)
        assert len(self.aaa._store.users) == 2, "The new user should be present"
        assert len(self.aaa._store.pending_registrations) == 0, \
            "The registration entry should be removed"



    @raises(AAAException)
    def test_send_password_reset_email_no_data(self):
        self.aaa.send_password_reset_email()

    @raises(AAAException)
    def test_send_password_reset_email_incorrect_data(self):
        self.aaa.send_password_reset_email(username='NotThere', email_addr='NoEmail')

    @raises(AAAException)
    def test_send_password_reset_email_incorrect_data2(self):
        # The username is valid but the email address is not matching
        self.aaa.send_password_reset_email(username='admin', email_addr='NoEmail')

    @raises(AAAException)
    def test_send_password_reset_email_only_incorrect_email(self):
        self.aaa.send_password_reset_email(email_addr='NoEmail')

    @raises(AAAException)
    def test_send_password_reset_email_only_incorrect_username(self):
        self.aaa.send_password_reset_email(username='NotThere')

    def test_send_password_reset_email_only_email(self):
        self.aaa.mailer.send_email = mock.Mock()
        self.aaa.send_password_reset_email(email_addr='admin@localhost.local',
            email_template='examples/views/password_reset_email')

    def test_send_password_reset_email_only_username(self):
        self.aaa.mailer.send_email = mock.Mock()
        self.aaa.send_password_reset_email(username='admin',
            email_template='examples/views/password_reset_email')



    @raises(AuthException)
    def test_perform_password_reset_invalid(self):
        self.aaa.reset_password('bogus', 'newpassword')


    @raises(AuthException)
    def test_perform_password_reset_timed_out(self):
        self.aaa.password_reset_timeout = 0
        token = self.aaa._reset_code('admin', 'admin@localhost.local')
        self.aaa.reset_password(token, 'newpassword')


    @raises(AAAException)
    def test_perform_password_reset_nonexistent_user(self):
        token = self.aaa._reset_code('admin_bogus', 'admin@localhost.local')
        self.aaa.reset_password(token, 'newpassword')


    # The following test should fail
    # an user can change the password reset timestamp by b64-decoding the token,
    # editing the field and b64-encoding it
    @SkipTest
    @raises(AuthException)
    def test_perform_password_reset_mangled_timestamp(self):
        token = self.aaa._reset_code('admin', 'admin@localhost.local')
        username, email_addr, tstamp, h = b64decode(token).split(':', 3)
        tstamp = str(int(tstamp) + 100)
        mangled_token = ':'.join((username, email_addr, tstamp, h))
        mangled_token = b64encode(mangled_token)
        self.aaa.reset_password(mangled_token, 'newpassword')


    @raises(AuthException)
    def test_perform_password_reset_mangled_username(self):
        token = self.aaa._reset_code('admin', 'admin@localhost.local')
        username, email_addr, tstamp, h = b64decode(token).split(':', 3)
        username += "mangled_username"
        mangled_token = ':'.join((username, email_addr, tstamp, h))
        mangled_token = b64encode(mangled_token)
        self.aaa.reset_password(mangled_token, 'newpassword')


    @raises(AuthException)
    def test_perform_password_reset_mangled_email(self):
        token = self.aaa._reset_code('admin', 'admin@localhost.local')
        username, email_addr, tstamp, h = b64decode(token).split(':', 3)
        email_addr += "mangled_email"
        mangled_token = ':'.join((username, email_addr, tstamp, h))
        mangled_token = b64encode(mangled_token)
        self.aaa.reset_password(mangled_token, 'newpassword')


    def test_perform_password_reset(self):
        token = self.aaa._reset_code('admin', 'admin@localhost.local')
        self.aaa.reset_password(token, 'newpassword')



REDIR = 302



class WebFunctional(object):

    def __init__(self):
        self._tmpdir = None
        self._app = None
        self._starting_dir = os.getcwd()

    def populate_conf_directory(self):
        """Populate a directory with valid configuration files, to be run just once
        The files are not modified by each test
        """
        self._tmpdir = os.path.join(self._tmproot, "cork_functional_test_source")

        # only do this once, as advertised
        if os.path.exists(self._tmpdir):
            return

        os.mkdir(self._tmpdir)
        os.mkdir(self._tmpdir + "/example_conf")

        cork = Cork(os.path.join(self._tmpdir, "example_conf"), initialize=True)

        cork._store.roles['admin'] = 100
        cork._store.roles['editor'] = 60
        cork._store.roles['user'] = 50
        cork._store.save_roles()

        tstamp = str(datetime.utcnow())
        username = password = 'admin'
        cork._store.users[username] = {
            'role': 'admin',
            'hash': cork._hash(username, password),
            'email_addr': username + '@localhost.local',
            'desc': username + ' test user',
            'creation_date': tstamp
        }
        username = password = ''
        cork._store.users[username] = {
            'role': 'user',
            'hash': cork._hash(username, password),
            'email_addr': username + '@localhost.local',
            'desc': username + ' test user',
            'creation_date': tstamp
        }
        cork._store.save_users()

    def populate_temp_dir(self):
        """populate the temporary test dir"""
        assert self._tmpdir is not None

        # copy the needed files
        shutil.copytree(
            os.path.join(self._starting_dir, 'tests/example_conf'),
            os.path.join(self._tmpdir, 'example_conf')
        )
        shutil.copytree(
            os.path.join(self._starting_dir, 'tests/views'),
            os.path.join(self._tmpdir, 'views')
        )

        print("Test directory set up")

    def remove_temp_dir(self):
        p = os.path.join(self._tmproot, 'cork_functional_test_wd')
        for f in glob.glob('%s*' % p):
            #shutil.rmtree(f)
            pass


    def teardown(self):
        print("Doing teardown")
        try:
            self._app.post('/logout')
        except:
            pass

        # drop the cookie
        self._app.reset()
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"
        # drop the cookie
        self._app.reset()

        #assert self._app.get('/admin').status != '200 OK'
        os.chdir(self._starting_dir)

        self._app.app.options['timeout'] = self._default_timeout
        self._app = None
        shutil.rmtree(self._tmpdir)
        self._tmpdir = None
        print("Teardown done")

    def setup(self):
        # create test dir and populate it using the example files

        # save the directory where the unit testing has been run
        if self._starting_dir is None:
            self._starting_dir = os.getcwd()

        # create json files to be used by Cork
        self._tmpdir = pick_temp_directory()
        assert self._tmpdir is not None
        self.populate_temp_dir()

        # change to the temporary test directory
        # cork relies on this being the current directory
        os.chdir(self._tmpdir)
        self.create_app_instance()
        self._app.reset()
        print("Reset done")
        self._default_timeout = self._app.app.options['timeout']
        print("Setup completed")



    # Utility functions


    def assert_200(self, path, match):
        """Assert that a page returns 200"""
        p = self._app.get(path)
        assert p.status_int == 200, "Status: %d, Location: %s" % \
            (p.status_int, p.location)

        if match is not None:
            assert match in p.body, "'%s' not found in body: '%s'" % (match, p.body)

        return p

    def assert_redirect(self, page, redir_page, post=None):
        """Assert that a page redirects to another one"""

        # perform GET or POST
        if post is None:
            p = self._app.get(page, status=302)
        else:
            assert isinstance(post, dict)
            p = self._app.post(page, post, status=302)

        dest = p.location.split(':80/')[-1]
        dest = "/%s" % dest
        assert dest == redir_page, "%s redirects to %s instead of %s" % \
            (page, dest, redir_page)

        return p

    # Tests

    def test_functional_login(self):
        assert self._app
        self._app.get('/admin', status=302)
        self._app.get('/my_role', status=302)

        self.login_as_admin()

        # fetch a page successfully
        r = self._app.get('/admin')
        assert r.status_int == 200, repr(r)

    def test_login_existing_user_none_password(self):
        p = self._app.post('/login', {'username': 'admin', 'password': None})
        assert p.status_int == REDIR, "Redirect expected"
        assert p.location == 'http://localhost:80/login', \
            "Incorrect redirect to %s" % p.location

    def test_login_nonexistent_user_none_password(self):
        p = self._app.post('/login', {'username': 'IAmNotHere', 'password': None})
        assert p.status_int == REDIR, "Redirect expected"
        assert p.location == 'http://localhost:80/login', \
            "Incorrect redirect to %s" % p.location

    def test_login_existing_user_empty_password(self):
        p = self._app.post('/login', {'username': 'admin', 'password': ''})
        assert p.status_int == REDIR, "Redirect expected"
        assert p.location == 'http://localhost:80/login', \
            "Incorrect redirect to %s" % p.location

    def test_login_nonexistent_user_empty_password(self):
        p = self._app.post('/login', {'username': 'IAmNotHere', 'password': ''})
        assert p.status_int == REDIR, "Redirect expected"
        assert p.location == 'http://localhost:80/login', \
            "Incorrect redirect to %s" % p.location

    def test_login_existing_user_wrong_password(self):
        p = self._app.post('/login', {'username': 'admin', 'password': 'BogusPassword'})
        assert p.status_int == REDIR, "Redirect expected"
        assert p.location == 'http://localhost:80/login', \
            "Incorrect redirect to %s" % p.location

    def test_functional_login_logout(self):
        # Incorrect login
        p = self._app.post('/login', {'username': 'admin', 'password': 'BogusPassword'})
        assert p.status_int == REDIR
        assert p.location == 'http://localhost:80/login', \
            "Incorrect redirect to %s" % p.location

        # log in and get a cookie
        p = self._app.post('/login', {'username': 'admin', 'password': 'admin'})
        assert p.status_int == REDIR
        assert p.location == 'http://localhost:80/', \
            "Incorrect redirect to %s" % p.location

        self.assert_200('/my_role', 'admin')

        # fetch a page successfully
        assert self._app.get('/admin').status_int == 200, "Admin page should be served"

        # log out
        assert self._app.get('/logout').status_int == REDIR

        # drop the cookie
        self._app.reset()
        assert self._app.cookies == {}, "The cookie should be gone"

        # fetch the same page, unsuccessfully
        assert self._app.get('/admin').status_int == REDIR

    def test_functional_user_creation_login_deletion(self):
        assert self._app.cookies == {}, "The cookie should be not set"

        # Log in as Admin
        p = self._app.post('/login', {'username': 'admin', 'password': 'admin'})
        assert p.status_int == REDIR
        assert p.location == 'http://localhost:80/', \
            "Incorrect redirect to %s" % p.location

        self.assert_200('/my_role', 'admin')

        username = 'BrandNewUser'

        # Delete the user
        ret = self._app.post('/delete_user', {
            'username': username,
        })

        # Create new user
        password = '42IsTheAnswer'
        ret = self._app.post('/create_user', {
            'username': username,
            'password': password,
            'role': 'user'
        })
        retj = json.loads(ret.body)
        assert 'ok' in retj and retj['ok'] == True, "Failed user creation: %s" % \
            ret.body

        # log out
        assert self._app.get('/logout').status_int == REDIR
        self._app.reset()
        assert self._app.cookies == {}, "The cookie should be gone"

        # Log in as user
        p = self._app.post('/login', {'username': username, 'password': password})
        assert p.status_int == REDIR and p.location == 'http://localhost:80/', \
            "Failed user login"

        # log out
        assert self._app.get('/logout').status_int == REDIR
        self._app.reset()
        assert self._app.cookies == {}, "The cookie should be gone"

        # Log in as user with empty password
        p = self._app.post('/login', {'username': username, 'password': ''})
        assert p.status_int == REDIR and p.location == 'http://localhost:80/login', \
            "User login should fail"
        assert self._app.cookies == {}, "The cookie should not be set"

        # Log in as Admin, again
        p = self._app.post('/login', {'username': 'admin', 'password': 'admin'})
        assert p.status_int == REDIR
        assert p.location == 'http://localhost:80/', \
            "Incorrect redirect to %s" % p.location

        self.assert_200('/my_role', 'admin')

        # Delete the user
        ret = self._app.post('/delete_user', {
            'username': username,
        })
        retj = json.loads(ret.body)
        assert 'ok' in retj and retj['ok'] == True, "Failed user deletion: %s" % \
            ret.body

    #def test_functional_user_registration(self):
    #    assert self._app.cookies == {}, "The cookie should be not set"
    #
    #    # Register new user
    #    username = 'BrandNewUser'
    #    password = '42IsTheAnswer'
    #    ret = self._app.post('/register', {
    #        'username': username,
    #        'password': password,
    #        'email_address': 'test@localhost.local'
    #    })


    def test_functional_expiration(self):
        raise NotImplementedError




# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt

"""
.. module:: backend.py
   :synopsis: Base Backend.
"""

class BackendIOException(Exception):
    """Generic Backend I/O Exception"""
    pass

def ni(*args, **kwargs):
    raise NotImplementedError

class Backend(object):
    """Base Backend class - to be subclassed by real backends."""
    save_users = ni
    save_roles = ni
    save_pending_registrations = ni

class Table(object):
    """Base Table class - to be subclassed by real backends."""
    __len__ = ni
    __contains__ = ni
    __setitem__ = ni
    __getitem__ = ni
    __iter__ = ni
    iteritems = ni




# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt

"""
.. module:: mongodb_backend
   :synopsis: MongoDB storage backend.
"""
from logging import getLogger
log = getLogger(__name__)

from .base_backend import Backend, Table

try:
    try:
        from pymongo import MongoClient
    except ImportError:  # pragma: no cover
        # Backward compatibility with PyMongo 2.2
        from pymongo import Connection as MongoClient

    pymongo_available = True
except ImportError:  # pragma: no cover
    pymongo_available = False



class MongoTable(Table):
    """Abstract MongoDB Table.
    Allow dictionary-like access.
    """
    def __init__(self, name, key_name, collection):
        self._name = name
        self._key_name = key_name
        self._coll = collection

    def create_index(self):
        """Create collection index."""
        self._coll.create_index(
            self._key_name,
            drop_dups=True,
            unique=True,
        )

    def __len__(self):
        return self._coll.count()

    def __contains__(self, value):
        r = self._coll.find_one({self._key_name: value})
        return r is not None

    def __iter__(self):
        """Iter on dictionary keys"""
        r = self._coll.find(fields=[self._key_name,])
        return (i[self._key_name] for i in r)

    def iteritems(self):
        """Iter on dictionary items.

        :returns: generator of (key, value) tuples
        """
        r = self._coll.find()
        for i in r:
            d = i.copy()
            d.pop(self._key_name)
            d.pop('_id')
            yield (i[self._key_name], d)

    def pop(self, key_val):
        """Remove a dictionary item"""
        r = self[key_val]
        self._coll.remove({self._key_name: key_val}, safe=True)
        return r


class MongoSingleValueTable(MongoTable):
    """MongoDB table accessible as a simple key -> value dictionary.
    Used to store roles.
    """
    # Values are stored in a MongoDB "column" named "val"
    def __init__(self, *args, **kw):
        super(MongoSingleValueTable, self).__init__(*args, **kw)

    def __setitem__(self, key_val, data):
        assert not isinstance(data, dict)
        spec = {self._key_name: key_val}
        data = {self._key_name: key_val, 'val': data}
        self._coll.update(spec, data, upsert=True, safe=True)

    def __getitem__(self, key_val):
        r = self._coll.find_one({self._key_name: key_val})
        if r is None:
            raise KeyError(key_val)

        return r['val']

class MongoMutableDict(dict):
    """Represent an item from a Table. Acts as a dictionary.
    """
    def __init__(self, parent, root_key, d):
        """Create a MongoMutableDict instance.
        :param parent: Table instance
        :type parent: :class:`MongoTable`
        """
        super(MongoMutableDict, self).__init__(d)
        self._parent = parent
        self._root_key = root_key

    def __setitem__(self, k, v):
        super(MongoMutableDict, self).__setitem__(k, v)
        self._parent[self._root_key] = self


class MongoMultiValueTable(MongoTable):
    """MongoDB table accessible as a dictionary.
    """
    def __init__(self, *args, **kw):
        super(MongoMultiValueTable, self).__init__(*args, **kw)

    def __setitem__(self, key_val, data):
        assert isinstance(data, dict)
        key_name = self._key_name
        if key_name in data:
            assert data[key_name] == key_val
        else:
            data[key_name] = key_val

        spec = {key_name: key_val}
        self._coll.update(spec, data, upsert=True)

    def __getitem__(self, key_val):
        r = self._coll.find_one({self._key_name: key_val})
        if r is None:
            raise KeyError(key_val)

        return MongoMutableDict(self, key_val, r)


class MongoDBBackend(Backend):
    def __init__(self, db_name='cork', hostname='localhost', port=27017, initialize=False, username=None, password=None):
        """Initialize MongoDB Backend"""
        connection = MongoClient(host=hostname, port=port)
        db = connection[db_name]
        if username and password:
            db.authenticate(username, password)
        self.users = MongoMultiValueTable('users', 'login', db.users)
        self.pending_registrations = MongoMultiValueTable(
            'pending_registrations',
            'pending_registration',
            db.pending_registrations
        )
        self.roles = MongoSingleValueTable('roles', 'role', db.roles)

        if initialize:
            self._initialize_storage()

    def _initialize_storage(self):
        """Create MongoDB indexes."""
        for c in (self.users, self.roles, self.pending_registrations):
            c.create_index()

    def save_users(self):
        pass

    def save_roles(self):
        pass

    def save_pending_registrations(self):
        pass



# Cork - Authentication module for the Flask web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Functional test using Json backend
#
# Requires WebTest http://webtest.pythonpaste.org/
#
# Run as: nosetests functional_test.py
#

from nose import SkipTest
from webtest import TestApp
from datetime import timedelta
import shutil
import os

import testutils
from cork import FlaskCork

REDIR = 302

class Test(testutils.WebFunctional):

    def create_app_instance(self):
        """create TestApp instance"""
        assert self._app is None
        import simple_webapp_flask
        self._bottle_app = simple_webapp_flask.app
        self._app = TestApp(self._bottle_app)
        #simple_webapp_flask.flask.session.secret_key = 'bogus'
        simple_webapp_flask.SECRET_KEY = 'bogus'
        print("Test App created")

    def teardown(self):
        print("Doing teardown")
        try:
            self._app.post('/logout')
        except:
            pass

        # drop the cookie
        self._app.reset()
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"
        # drop the cookie
        self._app.reset()

        #assert self._app.get('/admin').status != '200 OK'
        os.chdir(self._starting_dir)

        self._app.app.options['timeout'] = self._default_timeout
        self._app = None
        shutil.rmtree(self._tmpdir)
        self._tmpdir = None
        print("Teardown done")

    def setup(self):
        # create test dir and populate it using the example files

        # save the directory where the unit testing has been run
        if self._starting_dir is None:
            self._starting_dir = os.getcwd()

        # create json files to be used by Cork
        self._tmpdir = testutils.pick_temp_directory()
        assert self._tmpdir is not None
        self.populate_temp_dir()

        # change to the temporary test directory
        # cork relies on this being the current directory
        os.chdir(self._tmpdir)
        self.create_app_instance()
        self._app.reset()
        print("Reset done")
        #self._default_timeout = self._app.app.options['timeout']
        self._default_timeout = 30
        #FIXME: reset
        print("Setup completed")


    def login_as_admin(self):
        """perform log in"""
        assert self._app is not None
        assert 'session' not in self._app.cookies, "Unexpected cookie found"

        self.assert_200('/login', 'Please insert your credentials')
        assert 'session' not in self._app.cookies, "Unexpected cookie found"

        self.assert_redirect('/admin', '/sorry_page')

        self.assert_200('/user_is_anonymous', 'True')
        assert 'session' not in self._app.cookies, "Unexpected cookie found"

        post = {'username': 'admin', 'password': 'admin'}
        self.assert_redirect('/login', '/', post=post)
        assert 'session' in self._app.cookies, "Cookie not found"

        import bottle
        session = bottle.request.environ.get('beaker.session')
        print("Session from func. test", repr(session))

        self.assert_200('/login', 'Please insert your credentials')


        p = self._app.get('/admin')
        assert 'Welcome' in p.body, repr(p)

        p = self._app.get('/my_role', status=200)
        assert p.status_int == 200
        assert p.body == 'admin', "Sta"

        print("Login performed")


    def test_functional_expiration(self):
        self.login_as_admin()
        r = self._app.get('/admin')
        assert r.status_int == 200, repr(r)

        # change the cookie expiration in order to expire it
        saved = self._bottle_app.permanent_session_lifetime
        try:
            self._bottle_app.permanent_session_lifetime = timedelta(seconds=-1)

            # change the cookie expiration in order to expire it
            self._app.app.options['timeout'] = 0

            assert self._app.get('/admin').status_int == REDIR, "The cookie should have expired"


        finally:
            self._bottle_app.permanent_session_lifetime = saved






# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Unit testing
# Test scrypt-based password hashing
#

from nose import SkipTest
from nose.tools import assert_raises, raises, with_setup
from time import time
import os
import mock
import shutil

from cork import Cork, JsonBackend, AAAException, AuthException
import testutils


testdir = None  # Test directory
aaa = None  # global Cork instance
cookie_name = None  # global variable to track cookie status

tmproot = testutils.pick_temp_directory()


def setup_dir():
    """Setup test directory with valid JSON files"""
    global testdir
    tstamp = "%f" % time()
    testdir = "%s/fl_%s" % (tmproot, tstamp)
    os.mkdir(testdir)
    os.mkdir(testdir + '/views')
    with open("%s/users.json" % testdir, 'w') as f:
        f.write("""{"admin": {"email_addr": null, "desc": null, "role": "admin", "hash": "69f75f38ac3bfd6ac813794f3d8c47acc867adb10b806e8979316ddbf6113999b6052efe4ba95c0fa9f6a568bddf60e8e5572d9254dbf3d533085e9153265623", "creation_date": "2012-04-09 14:22:27.075596"}}""")
    with open("%s/roles.json" % testdir, 'w') as f:
        f.write("""{"special": 200, "admin": 100, "user": 50}""")
    with open("%s/register.json" % testdir, 'w') as f:
        f.write("""{}""")
    with open("%s/views/registration_email.tpl" % testdir, 'w') as f:
        f.write("""Username:{{username}} Email:{{email_addr}} Code:{{registration_code}}""")
    with open("%s/views/password_reset_email.tpl" % testdir, 'w') as f:
        f.write("""Username:{{username}} Email:{{email_addr}} Code:{{reset_code}}""")
    print("setup done in %s" % testdir)

def setUp():
    global aaa
    setup_dir()
    aaa = Cork(testdir, smtp_server='localhost', email_sender='test@localhost')

def teardown_dir():
    global cookie_name
    global testdir
    if testdir:
        shutil.rmtree(testdir)
        testdir = None
    cookie_name = None

def tearDown():
    global aaa
    aaa = None
    teardown_dir()


def test_password_hashing_scrypt():
    shash = aaa._hash('user_foo', 'bogus_pwd', algo='scrypt')
    assert len(shash) == 132, "hash length should be 132 and is %d" % len(shash)
    assert shash.endswith(b'='), "hash should end with '=': %r" % shash
    assert aaa._verify_password('user_foo', 'bogus_pwd', shash) == True, \
        "Hashing verification should succeed"


def test_password_hashing_scrypt_known_hash():
    salt = b's' * 32
    shash = aaa._hash('user_foo', 'bogus_pwd', salt=salt, algo='scrypt')
    assert shash == b'c3Nzc3Nzc3Nzc3Nzc3Nzc3Nzc3Nzc3Nzc3Nzc3Nzc3NzeLt/2Ta8vJOVqimNpN9G1WWxN1hxlUOJDPgH+0wqPpG20XQHFHLlksDIUo2BL4P8BMLBZj7F+cq6UP6pc304LQ==', repr(shash)


def test_password_hashing_scrypt_known_hash_2():
    salt = b'\0' * 32
    shash = aaa._hash('user_foo', 'bogus_pwd', salt=salt, algo='scrypt')
    assert shash == b'cwAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAmu5jQskr2/yX13Yxmc4TYL0MIuSxwo41SVJwn/QueiDdLGkNaEsxlKL37i98YofXxs8xJJAJlC3Xj/9Nx0RNBw=='


def test_password_hashing_scrypt_known_hash_3():
    salt = b'x' * 32
    shash = aaa._hash('user_foo', 'bogus_pwd', salt=salt, algo='scrypt')
    assert shash == b'c3h4eHh4eHh4eHh4eHh4eHh4eHh4eHh4eHh4eHh4eHh4yKuT1e8lovFZnaaOctivIvYBPkLoKDXX72kf5/nRuGIgyyhiKxxKE4LVYFKFCeVNPQM5m/+LulQkWhO0aB89lA=='


@raises(AssertionError)
def test_password_hashing_scrypt_incorrect_hash_len():
    salt = b'x' * 31 # Incorrect length
    shash = aaa._hash('user_foo', 'bogus_pwd', salt=salt, algo='scrypt')


def test_password_hashing_scrypt_incorrect_hash_value():
    shash = aaa._hash('user_foo', 'bogus_pwd', algo='scrypt')
    assert len(shash) == 132, "hash length should be 132 and is %d" % len(shash)
    assert shash.endswith(b'='), "hash should end with '='"
    assert aaa._verify_password('user_foo', '####', shash) == False, \
        "Hashing verification should fail"
    assert aaa._verify_password('###', 'bogus_pwd', shash) == False, \
        "Hashing verification should fail"



def test_password_hashing_scrypt_collision():
    salt = b'S' * 32
    hash1 = aaa._hash('user_foo', 'bogus_pwd', salt=salt, algo='scrypt')
    hash2 = aaa._hash('user_foobogus', '_pwd', salt=salt, algo='scrypt')
    assert hash1 != hash2, "Hash collision"





# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Unit testing - test the Cork module against a real MySQL instance
# running on localhost.

from nose import SkipTest
from nose.tools import assert_raises, raises, with_setup

from cork import Cork, AAAException, AuthException
from cork.backends import SqlAlchemyBackend
import os
import testutils


def connect_to_test_db():

    if os.environ.get('TRAVIS', False):
        # Using Travis-CI - https://travis-ci.org/
        password = ''
        db_name = 'myapp_test'
    else:
        password = ''
        db_name = 'cork_functional_test'

    uri = "mysql://root:%s@localhost/%s" % (password, db_name)
    return SqlAlchemyBackend(uri, initialize=True)

class Conf(object):

    def setup_test_db(self):

        mb = connect_to_test_db()

        ## Purge DB
        mb._drop_all_tables()
        #mb.users.empty_table()
        #mb.roles.empty_table()
        #mb.pending_registrations.empty_table()

        assert len(mb.roles) == 0
        assert len(mb.users) == 0

        # Create roles
        mb.roles.insert({'role': 'special', 'level': 200})
        mb.roles.insert({'role': 'admin', 'level': 100})
        mb.roles.insert({'role': 'editor', 'level': 60})
        mb.roles.insert({'role': 'user', 'level': 50})

        # Create admin
        mb.users.insert({
            "username": "admin",
            "email_addr": "admin@localhost.local",
            "desc": "admin test user",
            "role": "admin",
            "hash": "cLzRnzbEwehP6ZzTREh3A4MXJyNo+TV8Hs4//EEbPbiDoo+dmNg22f2RJC282aSwgyWv/O6s3h42qrA6iHx8yfw=",
            "creation_date": "2012-10-28 20:50:26.286723",
            "last_login": "2012-10-28 20:50:26.286723"
        })
        assert len(mb.roles) == 4
        assert len(mb.users) == 1

        return mb

    def purge_test_db(self):
        # Purge DB
        mb = connect_to_test_db()
        mb._drop_all_tables()


class TestMysqlUnauth(Conf, testutils.DatabaseInteractionAsUnauthenticated):
    pass

class TestMysqlAdmin(Conf, testutils.DatabaseInteractionAsAdmin):
    pass



# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Functional test using Json backend
#
# Requires WebTest http://webtest.pythonpaste.org/
#
# Run as: nosetests functional_test.py
#

from nose import SkipTest
from time import time
from datetime import datetime
from webtest import TestApp
import glob
import json
import os
import shutil

import testutils
from cork import Cork

REDIR = 302

class Test(testutils.WebFunctional):
    def __init__(self):
        self._tmpdir = None
        self._tmproot = None
        self._app = None
        self._starting_dir = os.getcwd()

    def populate_conf_directory(self):
        """Populate a directory with valid configuration files, to be run just once
        The files are not modified by each test
        """
        self._tmpdir = os.path.join(self._tmproot, "cork_functional_test_source")

        # only do this once, as advertised
        if os.path.exists(self._tmpdir):
            return

        os.mkdir(self._tmpdir)
        os.mkdir(self._tmpdir + "/example_conf")

        cork = Cork(os.path.join(self._tmpdir, "example_conf"), initialize=True)

        cork._store.roles['admin'] = 100
        cork._store.roles['editor'] = 60
        cork._store.roles['user'] = 50
        cork._store.save_roles()

        tstamp = str(datetime.utcnow())
        username = password = 'admin'
        cork._store.users[username] = {
            'role': 'admin',
            'hash': cork._hash(username, password),
            'email_addr': username + '@localhost.local',
            'desc': username + ' test user',
            'creation_date': tstamp
        }
        username = password = ''
        cork._store.users[username] = {
            'role': 'user',
            'hash': cork._hash(username, password),
            'email_addr': username + '@localhost.local',
            'desc': username + ' test user',
            'creation_date': tstamp
        }
        cork._store.save_users()

    def remove_temp_dir(self):
        p = os.path.join(self._tmproot, 'cork_functional_test_wd')
        for f in glob.glob('%s*' % p):
            #shutil.rmtree(f)
            pass

    @classmethod
    def setUpClass(cls):
        print("Setup class")

    def setup(self):
        # create test dir and populate it using the example files

        # save the directory where the unit testing has been run
        if self._starting_dir is None:
            self._starting_dir = os.getcwd()

        # create json files to be used by Cork
        self._tmproot = testutils.pick_temp_directory()
        assert self._tmproot is not None

        # purge the temporary test directory
        self.remove_temp_dir()

        self.populate_temp_dir()
        self.create_app_instance()
        self._app.reset()
        print("Reset done")
        print("Setup completed")

    def populate_temp_dir(self):
        """populate the temporary test dir"""
        assert self._tmproot is not None
        assert self._tmpdir is None

        tstamp = str(time())[5:]
        self._tmpdir = os.path.join(self._tmproot, "cork_functional_test_wd_%s" % tstamp)

        try:
            os.mkdir(self._tmpdir)
        except OSError:
            # The directory is already there, purge it
            print("Deleting %s" % self._tmpdir)
            shutil.rmtree(self._tmpdir)
            os.mkdir(self._tmpdir)

            #p = os.path.join(self._tmproot, 'cork_functional_test_wd')
            #for f in glob.glob('%s*' % p):
            #    shutil.rmtree(f)

        # copy the needed files
        shutil.copytree(
            os.path.join(self._starting_dir, 'tests/example_conf'),
            os.path.join(self._tmpdir, 'example_conf')
        )
        shutil.copytree(
            os.path.join(self._starting_dir, 'tests/views'),
            os.path.join(self._tmpdir, 'views')
        )

        # change to the temporary test directory
        # cork relies on this being the current directory
        os.chdir(self._tmpdir)

        print("Test directory set up")

    def create_app_instance(self):
        """create TestApp instance"""
        assert self._app is None
        import simple_webapp
        self._bottle_app = simple_webapp.app
        self._app = TestApp(self._bottle_app)
        print("Test App created")

    def teardown(self):
        print("Doing teardown")
        try:
            self._app.post('/logout')
        except:
            pass

        # drop the cookie
        self._app.reset()
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"
        # drop the cookie
        self._app.reset()

        #assert self._app.get('/admin').status != '200 OK'
        os.chdir(self._starting_dir)
        #if self._tmproot is not None:
        #    testutils.purge_temp_directory(self._tmproot)

        self._app.app.options['timeout'] = self._default_timeout
        self._app = None
        self._tmproot = None
        self._tmpdir = None
        print("Teardown done")

    def setup(self):
        # create test dir and populate it using the example files

        # save the directory where the unit testing has been run
        if self._starting_dir is None:
            self._starting_dir = os.getcwd()

        # create json files to be used by Cork
        self._tmproot = testutils.pick_temp_directory()
        assert self._tmproot is not None

        # purge the temporary test directory
        self.remove_temp_dir()

        self.populate_temp_dir()
        self.create_app_instance()
        self._app.reset()
        print("Reset done")
        self._default_timeout = self._app.app.options['timeout']
        print("Setup completed")

    def assert_200(self, path, match):
        """Assert that a page returns 200"""
        p = self._app.get(path)
        assert p.status_int == 200, "Status: %d, Location: %s" % \
            (p.status_int, p.location)

        if match is not None:
            assert match in p.body, "'%s' not found in body: '%s'" % (match, p.body)

        return p

    def assert_redirect(self, page, redir_page, post=None):
        """Assert that a page redirects to another one"""

        # perform GET or POST
        if post is None:
            p = self._app.get(page, status=302)
        else:
            assert isinstance(post, dict)
            p = self._app.post(page, post, status=302)

        dest = p.location.split(':80/')[-1]
        dest = "/%s" % dest
        assert dest == redir_page, "%s redirects to %s instead of %s" % \
            (page, dest, redir_page)

        return p

    def login_as_admin(self):
        """perform log in"""
        assert self._app is not None
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"

        self.assert_200('/login', 'Please insert your credentials')
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"

        self.assert_redirect('/admin', '/sorry_page')

        self.assert_200('/user_is_anonymous', 'True')
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"

        post = {'username': 'admin', 'password': 'admin'}
        self.assert_redirect('/login', '/', post=post)
        assert 'beaker.session.id' in self._app.cookies, "Cookie not found"

        import bottle
        session = bottle.request.environ.get('beaker.session')
        print("Session from func. test", repr(session))

        self.assert_200('/login', 'Please insert your credentials')


        p = self._app.get('/admin')
        assert 'Welcome' in p.body, repr(p)

        p = self._app.get('/my_role', status=200)
        assert p.status == '200 OK'
        assert p.body == 'admin', "Sta"

        print("Login performed")

    def test_functional_expiration(self):
        self.login_as_admin()
        r = self._app.get('/admin')
        assert r.status == '200 OK', repr(r)
        # change the cookie expiration in order to expire it
        self._app.app.options['timeout'] = 0
        assert self._app.get('/admin').status_int == REDIR, "The cookie should have expired"






# Cork - Authentication module for the Bottle web framework
# Copyright (C) 2013 Federico Ceratto and others, see AUTHORS file.
# Released under LGPLv3+ license, see LICENSE.txt
#
# Functional test using Json backend
#
# Requires WebTest http://webtest.pythonpaste.org/
#
# Run as: nosetests functional_test.py
#

from nose import SkipTest
from webtest import TestApp
import shutil
import os

import testutils
from cork import Cork

REDIR = 302

class Test(testutils.WebFunctional):

    def create_app_instance(self):
        """create TestApp instance"""
        assert self._app is None
        import simple_webapp
        self._bottle_app = simple_webapp.app
        self._app = TestApp(self._bottle_app)
        print("Test App created")




    def login_as_admin(self):
        """perform log in"""
        assert self._app is not None
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"

        self.assert_200('/login', 'Please insert your credentials')
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"

        self.assert_redirect('/admin', '/sorry_page')

        self.assert_200('/user_is_anonymous', 'True')
        assert 'beaker.session.id' not in self._app.cookies, "Unexpected cookie found"

        post = {'username': 'admin', 'password': 'admin'}
        self.assert_redirect('/login', '/', post=post)
        assert 'beaker.session.id' in self._app.cookies, "Cookie not found"

        import bottle
        session = bottle.request.environ.get('beaker.session')
        print("Session from func. test", repr(session))

        self.assert_200('/login', 'Please insert your credentials')


        p = self._app.get('/admin')
        assert 'Welcome' in p.body, repr(p)

        p = self._app.get('/my_role', status=200)
        assert p.status_int == 200
        assert p.body == 'admin', "Sta"

        print("Login performed")



    def test_functional_expiration(self):
        self.login_as_admin()
        r = self._app.get('/admin')
        assert r.status == '200 OK', repr(r)
        # change the cookie expiration in order to expire it
        self._app.app.options['timeout'] = 0
        assert self._app.get('/admin').status_int == REDIR, "The cookie should have expired"




#!/usr/bin/env python

from setuptools import setup

__version__ = '0.11.1'

CLASSIFIERS = map(str.strip,
"""Environment :: Console
License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)
Natural Language :: English
Operating System :: POSIX :: Linux
Programming Language :: Python
Topic :: Internet :: WWW/HTTP :: WSGI
Topic :: Security
Topic :: Software Development :: Libraries :: Python Modules
""".splitlines())

setup(
    name="bottle-cork",
    version=__version__,
    author="Federico Ceratto",
    author_email="federico.ceratto@gmail.com",
    description="Authentication/Authorization library for Bottle",
    license="LGPLv3+",
    url="http://cork.firelet.net/",
    long_description="Cork is a simple Authentication/Authorization library" \
        "for the Bottle web framework.",
    classifiers=CLASSIFIERS,
    install_requires=[
        'Beaker',
        'Bottle',
        'pycrypto',
    ],
    extras_require = {
        'scrypt': ["scrypt>=0.6.1"],
    },
    packages=['cork'],
    platforms=['Linux'],
    test_suite='nose.collector',
    tests_require=['nose'],
)



import json
import base64
import hmac
from Crypto.Cipher import AES

def _strcmp(a, b):
    """Compares two strings while preventing timing attacks. Execution time
    is not affected by lenghth of common prefix on strings of the same length"""
    return not sum(0 if x==y else 1 for x, y in zip(a, b)) and len(a) == len(b)

class SecureSession(object):

    def __init__(self):

    json()
    
    


    
        
       base64.b64encode(hmac.new(tob(key), msg).digest())):
            return pickle.loads(base64.b64decode(msg))
